@@ -23,8 +23,8 @@
 #include <gnutls_errors.h>
 #include <gnutls_int.h>
 #include <gnutls/crypto.h>
-#include <accelerated/cryptodev.h>
 #include <gnutls_errors.h>
+#include <accelerated/cryptodev.h>
 
 #ifdef ENABLE_CRYPTODEV
 
@@ -40,16 +40,15 @@
 #define EALG_MAX_BLOCK_LEN 16
 #endif
 
-static int cryptodev_fd = -1;
+int _gnutls_cryptodev_fd = -1;
 
-static int register_mac (int cfd);
+static int register_mac_digest (int cfd);
 
 struct cryptodev_ctx
 {
   struct session_op sess;
   struct crypt_op cryp;
   uint8_t iv[EALG_MAX_BLOCK_LEN];
-  uint8_t key[CRYPTO_CIPHER_MAX_KEY_LEN];
   int cfd;
 };
 
@@ -64,7 +63,7 @@ static const int gnutls_cipher_map[] = {
 };
 
 static int
-cryptodev_cipher_init (gnutls_cipher_algorithm_t algorithm, void **_ctx)
+cryptodev_cipher_init (gnutls_cipher_algorithm_t algorithm, void **_ctx, int enc)
 {
   struct cryptodev_ctx *ctx;
   int cipher = gnutls_cipher_map[algorithm];
@@ -78,9 +77,8 @@ cryptodev_cipher_init (gnutls_cipher_algorithm_t algorithm, void **_ctx)
 
   ctx = *_ctx;
 
-  ctx->cfd = cryptodev_fd;
+  ctx->cfd = _gnutls_cryptodev_fd;
   ctx->sess.cipher = cipher;
-  ctx->sess.key = ctx->key;
   ctx->cryp.iv = ctx->iv;
 
   return 0;
@@ -92,7 +90,7 @@ cryptodev_cipher_setkey (void *_ctx, const void *key, size_t keysize)
   struct cryptodev_ctx *ctx = _ctx;
 
   ctx->sess.keylen = keysize;
-  memcpy (ctx->key, key, keysize);
+  ctx->sess.key = (void*)key;
 
   if (ioctl (ctx->cfd, CIOCGSESSION, &ctx->sess))
     {
@@ -123,6 +121,7 @@ cryptodev_encrypt (void *_ctx, const void *plain, size_t plainsize,
   ctx->cryp.src = (void *) plain;
   ctx->cryp.dst = encr;
   ctx->cryp.op = COP_ENCRYPT;
+
   if (ioctl (ctx->cfd, CIOCCRYPT, &ctx->cryp))
     {
       gnutls_assert ();
@@ -147,7 +146,6 @@ cryptodev_decrypt (void *_ctx, const void *encr, size_t encrsize,
       return GNUTLS_E_CRYPTODEV_IOCTL_ERROR;
     }
   return 0;
-
 }
 
 static void
@@ -155,7 +153,7 @@ cryptodev_deinit (void *_ctx)
 {
   struct cryptodev_ctx *ctx = _ctx;
 
-  ioctl (ctx->cfd, CIOCFSESSION, &ctx->sess);
+  ioctl (ctx->cfd, CIOCFSESSION, &ctx->sess.ses);
   gnutls_free (ctx);
 }
 
@@ -172,8 +170,14 @@ static int
 register_crypto (int cfd)
 {
   struct session_op sess;
-  char fake_key[CRYPTO_CIPHER_MAX_KEY_LEN];
-  int i = 0, ret;
+  uint8_t fake_key[CRYPTO_CIPHER_MAX_KEY_LEN];
+  unsigned int i;
+  int ret;
+#ifdef CIOCGSESSINFO
+  struct session_info_op siop;
+
+  memset(&siop, 0, sizeof(siop));
+#endif
 
   memset (&sess, 0, sizeof (sess));
 
@@ -193,7 +197,19 @@ register_crypto (int cfd)
           continue;
         }
 
-      ioctl (cfd, CIOCFSESSION, &sess);
+#ifdef CIOCGSESSINFO
+      siop.ses = sess.ses; /* do not register ciphers that are not hw accelerated */
+      if (ioctl(cfd, CIOCGSESSINFO, &siop) == 0) 
+        {
+          if (!(siop.flags & SIOP_FLAG_KERNEL_DRIVER_ONLY))
+            {
+              ioctl (cfd, CIOCFSESSION, &sess.ses);
+              continue;
+            }
+        }
+#endif
+
+      ioctl (cfd, CIOCFSESSION, &sess.ses);
 
       _gnutls_debug_log ("/dev/crypto: registering: %s\n",
                          gnutls_cipher_get_name (i));
@@ -206,7 +222,7 @@ register_crypto (int cfd)
 
     }
 
-  return 0;
+  return _cryptodev_register_gcm_crypto(cfd);
 }
 
 int
@@ -215,8 +231,8 @@ _gnutls_cryptodev_init (void)
   int ret;
 
   /* Open the crypto device */
-  cryptodev_fd = open ("/dev/crypto", O_RDWR, 0);
-  if (cryptodev_fd < 0)
+  _gnutls_cryptodev_fd = open ("/dev/crypto", O_RDWR, 0);
+  if (_gnutls_cryptodev_fd < 0)
     {
       gnutls_assert ();
       return GNUTLS_E_CRYPTODEV_DEVICE_ERROR;
@@ -226,7 +242,7 @@ _gnutls_cryptodev_init (void)
   {
     int cfd = -1;
     /* Clone file descriptor */
-    if (ioctl (cryptodev_fd, CRIOGET, &cfd))
+    if (ioctl (_gnutls_cryptodev_fd, CRIOGET, &cfd))
       {
         gnutls_assert ();
         return GNUTLS_E_CRYPTODEV_IOCTL_ERROR;
@@ -239,18 +255,18 @@ _gnutls_cryptodev_init (void)
         return GNUTLS_E_CRYPTODEV_IOCTL_ERROR;
       }
 
-    close (cryptodev_fd);
-    cryptodev_fd = cfd;
+    close (_gnutls_cryptodev_fd);
+    _gnutls_cryptodev_fd = cfd;
   }
 #endif
 
-  ret = register_crypto (cryptodev_fd);
+  ret = register_crypto (_gnutls_cryptodev_fd);
   if (ret < 0)
     gnutls_assert ();
 
   if (ret >= 0)
     {
-      ret = register_mac (cryptodev_fd);
+      ret = register_mac_digest (_gnutls_cryptodev_fd);
       if (ret < 0)
         gnutls_assert ();
     }
@@ -258,7 +274,7 @@ _gnutls_cryptodev_init (void)
   if (ret < 0)
     {
       gnutls_assert ();
-      close (cryptodev_fd);
+      close (_gnutls_cryptodev_fd);
     }
 
   return ret;
@@ -267,14 +283,14 @@ _gnutls_cryptodev_init (void)
 void
 _gnutls_cryptodev_deinit (void)
 {
-  close (cryptodev_fd);
+  if (_gnutls_cryptodev_fd != -1) close (_gnutls_cryptodev_fd);
 }
 
 /* MAC and digest stuff */
 
 /* if we are using linux /dev/crypto
  */
-#if defined COP_FLAG_UPDATE
+#if defined(COP_FLAG_UPDATE) && defined(COP_FLAG_RESET)
 
 static const int gnutls_mac_map[] = {
   [GNUTLS_MAC_MD5] = CRYPTO_MD5_HMAC,
@@ -299,10 +315,9 @@ cryptodev_mac_init (gnutls_mac_algorithm_t algorithm, void **_ctx)
 
   ctx = *_ctx;
 
-  ctx->cfd = cryptodev_fd;
+  ctx->cfd = _gnutls_cryptodev_fd;
 
   ctx->sess.mac = mac;
-  ctx->sess.mackey = ctx->key;
 
   return 0;
 }
@@ -313,7 +328,7 @@ cryptodev_mac_setkey (void *_ctx, const void *key, size_t keysize)
   struct cryptodev_ctx *ctx = _ctx;
 
   ctx->sess.mackeylen = keysize;
-  memcpy (ctx->key, key, keysize);
+  ctx->sess.mackey = (void*)key;
 
   if (ioctl (ctx->cfd, CIOCGSESSION, &ctx->sess))
     {
@@ -329,11 +344,13 @@ static int
 cryptodev_mac_hash (void *_ctx, const void *text, size_t textsize)
 {
   struct cryptodev_ctx *ctx = _ctx;
+
   ctx->cryp.len = textsize;
   ctx->cryp.src = (void *) text;
   ctx->cryp.dst = NULL;
   ctx->cryp.op = COP_ENCRYPT;
   ctx->cryp.flags = COP_FLAG_UPDATE;
+  
   if (ioctl (ctx->cfd, CIOCCRYPT, &ctx->cryp))
     {
       gnutls_assert ();
@@ -346,11 +363,13 @@ static int
 cryptodev_mac_output (void *_ctx, void *digest, size_t digestsize)
 {
   struct cryptodev_ctx *ctx = _ctx;
+
   ctx->cryp.len = 0;
   ctx->cryp.src = NULL;
   ctx->cryp.mac = digest;
   ctx->cryp.op = COP_ENCRYPT;
   ctx->cryp.flags = COP_FLAG_FINAL;
+
   if (ioctl (ctx->cfd, CIOCCRYPT, &ctx->cryp))
     {
       gnutls_assert ();
@@ -360,6 +379,55 @@ cryptodev_mac_output (void *_ctx, void *digest, size_t digestsize)
   return 0;
 }
 
+static void
+cryptodev_mac_reset (void *_ctx)
+{
+  struct cryptodev_ctx *ctx = _ctx;
+
+  ctx->cryp.len = 0;
+  ctx->cryp.src = NULL;
+  ctx->cryp.dst = NULL;
+  ctx->cryp.op = COP_ENCRYPT;
+  ctx->cryp.flags = COP_FLAG_RESET;
+  
+  ioctl (ctx->cfd, CIOCCRYPT, &ctx->cryp);
+}
+
+static int
+cryptodev_mac_fast (gnutls_mac_algorithm_t algo,
+                    const void *key, size_t key_size, const void *text,
+                    size_t text_size, void *digest)
+{
+struct cryptodev_ctx ctx;
+int ret;
+
+  memset(&ctx, 0, sizeof(ctx));
+  ctx.cfd = _gnutls_cryptodev_fd;
+  ctx.sess.mac = gnutls_mac_map[algo];
+
+  ctx.sess.mackeylen = key_size;
+  ctx.sess.mackey = (void*)key;
+
+  if (ioctl (ctx.cfd, CIOCGSESSION, &ctx.sess))
+    return gnutls_assert_val(GNUTLS_E_CRYPTODEV_IOCTL_ERROR);
+
+  ctx.cryp.ses = ctx.sess.ses;
+
+  ctx.cryp.len = text_size;
+  ctx.cryp.src = (void *) text;
+  ctx.cryp.dst = NULL;
+  ctx.cryp.op = COP_ENCRYPT;
+  ctx.cryp.mac = digest;
+  
+  ret = ioctl (ctx.cfd, CIOCCRYPT, &ctx.cryp);
+
+  ioctl (_gnutls_cryptodev_fd, CIOCFSESSION, &ctx.sess.ses);
+  if (ret != 0)
+    return gnutls_assert_val(GNUTLS_E_CRYPTODEV_IOCTL_ERROR);
+  
+  return 0;
+}
+
 #define cryptodev_mac_deinit cryptodev_deinit
 
 static const gnutls_crypto_mac_st mac_struct = {
@@ -367,15 +435,109 @@ static const gnutls_crypto_mac_st mac_struct = {
   .setkey = cryptodev_mac_setkey,
   .hash = cryptodev_mac_hash,
   .output = cryptodev_mac_output,
-  .deinit = cryptodev_mac_deinit
+  .deinit = cryptodev_mac_deinit,
+  .reset = cryptodev_mac_reset,
+  .fast = cryptodev_mac_fast
+};
+
+/* Digest algorithms */
+
+static const int gnutls_digest_map[] = {
+  [GNUTLS_DIG_MD5] = CRYPTO_MD5,
+  [GNUTLS_DIG_SHA1] = CRYPTO_SHA1,
+  [GNUTLS_DIG_SHA256] = CRYPTO_SHA2_256,
+  [GNUTLS_DIG_SHA384] = CRYPTO_SHA2_384,
+  [GNUTLS_DIG_SHA512] = CRYPTO_SHA2_512,
 };
 
 static int
-register_mac (int cfd)
+cryptodev_digest_init (gnutls_digest_algorithm_t algorithm, void **_ctx)
+{
+  struct cryptodev_ctx *ctx;
+  int dig = gnutls_digest_map[algorithm];
+
+  *_ctx = gnutls_calloc (1, sizeof (struct cryptodev_ctx));
+  if (*_ctx == NULL)
+    {
+      gnutls_assert ();
+      return GNUTLS_E_MEMORY_ERROR;
+    }
+
+  ctx = *_ctx;
+
+  ctx->cfd = _gnutls_cryptodev_fd;
+  ctx->sess.mac = dig;
+
+  if (ioctl (ctx->cfd, CIOCGSESSION, &ctx->sess))
+    {
+      gnutls_assert ();
+      gnutls_free(ctx);
+      return GNUTLS_E_CRYPTODEV_IOCTL_ERROR;
+    }
+  ctx->cryp.ses = ctx->sess.ses;
+
+  return 0;
+}
+
+#define cryptodev_digest_hash cryptodev_mac_hash
+#define cryptodev_digest_output cryptodev_mac_output
+#define cryptodev_digest_reset cryptodev_mac_reset
+
+static int
+cryptodev_digest_fast (gnutls_digest_algorithm_t algo,
+                       const void *text, size_t text_size, 
+                       void *digest)
+{
+struct cryptodev_ctx ctx;
+int ret;
+
+  memset(&ctx, 0, sizeof(ctx));
+  ctx.cfd = _gnutls_cryptodev_fd;
+  ctx.sess.mac = gnutls_digest_map[algo];
+
+  if (ioctl (ctx.cfd, CIOCGSESSION, &ctx.sess))
+    return gnutls_assert_val(GNUTLS_E_CRYPTODEV_IOCTL_ERROR);
+
+  ctx.cryp.ses = ctx.sess.ses;
+
+  ctx.cryp.len = text_size;
+  ctx.cryp.src = (void *) text;
+  ctx.cryp.dst = NULL;
+  ctx.cryp.op = COP_ENCRYPT;
+  ctx.cryp.mac = digest;
+  
+  ret = ioctl (ctx.cfd, CIOCCRYPT, &ctx.cryp);
+  
+  ioctl (_gnutls_cryptodev_fd, CIOCFSESSION, &ctx.sess.ses);
+  if (ret != 0)
+    return gnutls_assert_val(GNUTLS_E_CRYPTODEV_IOCTL_ERROR);
+  
+  return 0;
+}
+
+#define cryptodev_digest_deinit cryptodev_deinit
+
+static const gnutls_crypto_digest_st digest_struct = {
+  .init = cryptodev_digest_init,
+  .hash = cryptodev_digest_hash,
+  .output = cryptodev_digest_output,
+  .deinit = cryptodev_digest_deinit,
+  .reset = cryptodev_digest_reset,
+  .fast = cryptodev_digest_fast
+};
+
+static int
+register_mac_digest (int cfd)
 {
   struct session_op sess;
-  char fake_key[CRYPTO_CIPHER_MAX_KEY_LEN];
-  int i = 0, ret;
+  uint8_t fake_key[CRYPTO_CIPHER_MAX_KEY_LEN];
+  unsigned int i;
+  int ret;
+#ifdef CIOCGSESSINFO
+  struct session_info_op siop;
+
+  memset(&siop, 0, sizeof(siop));
+#endif
 
   memset (&sess, 0, sizeof (sess));
   for (i = 0; i < sizeof (gnutls_mac_map) / sizeof (gnutls_mac_map[0]); i++)
@@ -392,17 +554,65 @@ register_mac (int cfd)
           continue;
         }
 
-      ioctl (cfd, CIOCFSESSION, &sess);
-
-      _gnutls_debug_log ("/dev/crypto: registering: %s\n",
+#ifdef CIOCGSESSINFO
+      siop.ses = sess.ses; /* do not register ciphers that are not hw accelerated */
+      if (ioctl(cfd, CIOCGSESSINFO, &siop) == 0) 
+        {
+          if (!(siop.flags & SIOP_FLAG_KERNEL_DRIVER_ONLY))
+            {
+              ioctl (cfd, CIOCFSESSION, &sess.ses);
+              continue;
+            }
+        }
+#endif
+      _gnutls_debug_log ("/dev/crypto: registering: HMAC-%s\n",
                          gnutls_mac_get_name (i));
+
+      ioctl (cfd, CIOCFSESSION, &sess.ses);
+
       ret = gnutls_crypto_single_mac_register (i, 90, &mac_struct);
       if (ret < 0)
         {
           gnutls_assert ();
           return ret;
         }
+    }
 
+  memset (&sess, 0, sizeof (sess));
+  for (i = 0; i < sizeof (gnutls_digest_map) / sizeof (gnutls_digest_map[0]); i++)
+    {
+      if (gnutls_digest_map[i] == 0)
+        continue;
+
+      sess.mac = gnutls_digest_map[i];
+
+      if (ioctl (cfd, CIOCGSESSION, &sess))
+        {
+          continue;
+        }
+
+#ifdef CIOCGSESSINFO
+      siop.ses = sess.ses;
+      if (ioctl(cfd, CIOCGSESSINFO, &siop) == 0) 
+        {
+          if (!(siop.flags & SIOP_FLAG_KERNEL_DRIVER_ONLY))
+            {
+              ioctl (cfd, CIOCFSESSION, &sess.ses);
+              continue;
+            }
+        }
+#endif
+
+      ioctl (cfd, CIOCFSESSION, &sess.ses);
+
+      _gnutls_debug_log ("/dev/crypto: registering: %s\n",
+                         gnutls_mac_get_name (i));
+      ret = gnutls_crypto_single_digest_register (i, 90, &digest_struct);
+      if (ret < 0)
+        {
+          gnutls_assert ();
+          return ret;
+        }
     }
 
   return 0;
@@ -410,22 +620,22 @@ register_mac (int cfd)
 
 #else
 static int
-register_mac (int cfd)
+register_mac_digest (int cfd)
 {
   return 0;
 }
 
-#endif /* defined(CIOCSESSIONCLONE) && defined(COP_FLAG_UPDATE) */
+#endif /* defined(COP_FLAG_UPDATE) */
 
 #else /* ENABLE_CRYPTODEV */
 int
-_gnutls_cryptodev_init ()
+_gnutls_cryptodev_init (void)
 {
   return 0;
 }
 
 void
-_gnutls_cryptodev_deinit ()
+_gnutls_cryptodev_deinit (void)
 {
   return;
 }
