@@ -409,13 +409,17 @@ compressed_to_ciphertext (gnutls_session_t session,
     }
 
   /* add the authenticate data */
-  _gnutls_auth_cipher_add_auth(&params->write.cipher_state, preamble, preamble_size);
+  ret = _gnutls_auth_cipher_add_auth(&params->write.cipher_state, preamble, preamble_size);
+  if (ret < 0)
+    return gnutls_assert_val(ret);
 
   /* Actual encryption (inplace).
    */
   ret =
-    _gnutls_auth_cipher_encrypt_tag (&params->write.cipher_state,
-      cipher_data, length_to_encrypt, tag_ptr, tag_size, compressed->size);
+    _gnutls_auth_cipher_encrypt2_tag (&params->write.cipher_state,
+        cipher_data, length_to_encrypt, 
+        cipher_data, cipher_size,
+        tag_ptr, tag_size, compressed->size);
   if (ret < 0)
     return gnutls_assert_val(ret);
 
@@ -493,11 +497,14 @@ ciphertext_to_compressed (gnutls_session_t session,
         make_preamble (UINT64DATA(*sequence), type,
                        length, ver, preamble);
 
-      _gnutls_auth_cipher_add_auth (&params->read.cipher_state, preamble, preamble_size);
+      ret = _gnutls_auth_cipher_add_auth (&params->read.cipher_state, preamble, preamble_size);
+      if (ret < 0)
+        return gnutls_assert_val(ret);
 
       if ((ret =
-           _gnutls_auth_cipher_decrypt (&params->read.cipher_state,
-             ciphertext->data, length_to_decrypt)) < 0)
+           _gnutls_auth_cipher_decrypt2 (&params->read.cipher_state,
+             ciphertext->data, length_to_decrypt,
+             ciphertext->data, ciphertext->size)) < 0)
         return gnutls_assert_val(ret);
 
       break;
@@ -573,8 +580,13 @@ ciphertext_to_compressed (gnutls_session_t session,
       preamble_size =
         make_preamble (UINT64DATA(*sequence), type,
                        length, ver, preamble);
-      _gnutls_auth_cipher_add_auth (&params->read.cipher_state, preamble, preamble_size);
-      _gnutls_auth_cipher_add_auth (&params->read.cipher_state, ciphertext->data, length);
+      ret = _gnutls_auth_cipher_add_auth (&params->read.cipher_state, preamble, preamble_size);
+      if (ret < 0)
+        return gnutls_assert_val(ret);
+
+      ret = _gnutls_auth_cipher_add_auth (&params->read.cipher_state, ciphertext->data, length);
+      if (ret < 0)
+        return gnutls_assert_val(ret);
 
       break;
     default:
