@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Free Software Foundation, Inc.
+ * Copyright (C) 2011-2012 Free Software Foundation, Inc.
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -71,7 +71,7 @@ _gnutls_ecc_ansi_x963_export (gnutls_ecc_curve_t curve, bigint_t x, bigint_t y,
 
 
 int
-_gnutls_ecc_ansi_x963_import (const opaque * in,
+_gnutls_ecc_ansi_x963_import (const uint8_t * in,
                               unsigned long inlen, bigint_t * x, bigint_t * y)
 {
   int ret;
