@@ -64,7 +64,7 @@
 #define FALSE 0
 
 static int _gnutls_server_select_comp_method (gnutls_session_t session,
-                                              opaque * data, int datalen);
+                                              uint8_t * data, int datalen);
 static int
 _gnutls_remove_unwanted_ciphersuites (gnutls_session_t session,
                                       uint8_t * cipher_suites,
@@ -88,17 +88,17 @@ _gnutls_handshake_hash_buffer_empty (gnutls_session_t session)
 static int
 _gnutls_handshake_hash_add_recvd (gnutls_session_t session,
                                   gnutls_handshake_description_t recv_type,
-                                  opaque * header, uint16_t header_size,
-                                  opaque * dataptr, uint32_t datalen);
+                                  uint8_t * header, uint16_t header_size,
+                                  uint8_t * dataptr, uint32_t datalen);
 
 static int
 _gnutls_handshake_hash_add_sent (gnutls_session_t session,
                                  gnutls_handshake_description_t type,
-                                 opaque * dataptr, uint32_t datalen);
+                                 uint8_t * dataptr, uint32_t datalen);
 
 static int
 _gnutls_recv_hello_verify_request (gnutls_session_t session,
-				   opaque * data, int datalen);
+				   uint8_t * data, int datalen);
 
 
 /* Clears the handshake hash buffers and handles.
@@ -180,7 +180,7 @@ _gnutls_set_client_random (gnutls_session_t session, uint8_t * rnd)
 #define SSL3_SERVER_MSG "SRVR"
 #define SSL_MSG_LEN 4
 static int
-_gnutls_ssl3_finished (gnutls_session_t session, int type, opaque * ret, int sending)
+_gnutls_ssl3_finished (gnutls_session_t session, int type, uint8_t * ret, int sending)
 {
   digest_hd_st td_md5;
   digest_hd_st td_sha;
@@ -247,7 +247,7 @@ static int
 _gnutls_finished (gnutls_session_t session, int type, void *ret, int sending)
 {
   const int siz = TLS_MSG_LEN;
-  opaque concat[MAX_HASH_SIZE + 16 /*MD5 */ ];
+  uint8_t concat[MAX_HASH_SIZE + 16 /*MD5 */ ];
   size_t hash_len;
   const char *mesg;
   int rc, len;
@@ -297,7 +297,7 @@ _gnutls_finished (gnutls_session_t session, int type, void *ret, int sending)
  * and put it to dst.
  */
 int
-_gnutls_tls_create_random (opaque * dst)
+_gnutls_tls_create_random (uint8_t * dst)
 {
   uint32_t tim;
   int ret;
@@ -387,7 +387,7 @@ _gnutls_user_hello_func (gnutls_session_t session,
  * since SSL version 2.0 is not supported).
  */
 static int
-_gnutls_read_client_hello (gnutls_session_t session, opaque * data,
+_gnutls_read_client_hello (gnutls_session_t session, uint8_t * data,
                            int datalen)
 {
   uint8_t session_id_len;
@@ -396,7 +396,7 @@ _gnutls_read_client_hello (gnutls_session_t session, opaque * data,
   gnutls_protocol_t adv_version;
   int neg_version;
   int len = datalen;
-  opaque rnd[GNUTLS_RANDOM_SIZE], *suite_ptr, *comp_ptr, *session_id;
+  uint8_t rnd[GNUTLS_RANDOM_SIZE], *suite_ptr, *comp_ptr, *session_id;
 
   DECR_LEN (len, 2);
 
@@ -598,7 +598,7 @@ static int
 _gnutls_send_finished (gnutls_session_t session, int again)
 {
   mbuffer_st *bufel;
-  opaque *data;
+  uint8_t *data;
   int ret;
   size_t vdata_size = 0;
 
@@ -678,7 +678,7 @@ _gnutls_recv_finished (gnutls_session_t session)
 
   ret =
     _gnutls_recv_handshake (session, GNUTLS_HANDSHAKE_FINISHED, 
-      MANDATORY_PACKET, &buf);
+                            0, &buf);
   if (ret < 0)
     {
       ERR ("recv finished int", ret);
@@ -765,14 +765,14 @@ cleanup:
  * RSA algorithms, PK_DSA if DSS, and PK_ANY for both or PK_NONE for none.
  */
 static int
-server_find_pk_algos_in_ciphersuites (const opaque *
+server_find_pk_algos_in_ciphersuites (const uint8_t *
                                               data, unsigned int datalen,
                                               gnutls_pk_algorithm_t * algos,
                                               size_t* algos_size)
 {
   unsigned int j;
   gnutls_kx_algorithm_t kx;
-  int max = *algos_size;
+  unsigned int max = *algos_size;
 
   if (datalen % 2 != 0)
     {
@@ -800,10 +800,11 @@ server_find_pk_algos_in_ciphersuites (const opaque *
  * it adds the suite to the session and performs some checks.
  */
 int
-_gnutls_server_select_suite (gnutls_session_t session, opaque * data,
-                             int datalen)
+_gnutls_server_select_suite (gnutls_session_t session, uint8_t * data,
+                             unsigned int datalen)
 {
-  int i, j, ret, cipher_suites_size;
+  int ret;
+  unsigned int i, j, cipher_suites_size;
   size_t pk_algos_size;
   uint8_t cipher_suites[MAX_CIPHERSUITE_SIZE];
   int retval, err;
@@ -815,7 +816,7 @@ _gnutls_server_select_suite (gnutls_session_t session, opaque * data,
    */
   if (session->internals.priorities.sr != SR_DISABLED)
     {
-      int offset;
+      unsigned int offset;
 
       for (offset = 0; offset < datalen; offset += 2)
         {
@@ -976,7 +977,7 @@ finish:
  */
 static int
 _gnutls_server_select_comp_method (gnutls_session_t session,
-                                   opaque * data, int datalen)
+                                   uint8_t * data, int datalen)
 {
   int x, i, j;
   uint8_t comps[MAX_ALGOS];
@@ -1185,8 +1186,8 @@ _gnutls_send_handshake (gnutls_session_t session, mbuffer_st * bufel,
 static int
 _gnutls_handshake_hash_add_recvd (gnutls_session_t session,
                                   gnutls_handshake_description_t recv_type,
-                                  opaque * header, uint16_t header_size,
-                                  opaque * dataptr, uint32_t datalen)
+                                  uint8_t * header, uint16_t header_size,
+                                  uint8_t * dataptr, uint32_t datalen)
 {
   int ret;
 
@@ -1219,7 +1220,7 @@ _gnutls_handshake_hash_add_recvd (gnutls_session_t session,
 static int
 _gnutls_handshake_hash_add_sent (gnutls_session_t session,
                                  gnutls_handshake_description_t type,
-                                 opaque * dataptr, uint32_t datalen)
+                                 uint8_t * dataptr, uint32_t datalen)
 {
   int ret;
 
@@ -1250,25 +1251,21 @@ _gnutls_handshake_hash_add_sent (gnutls_session_t session,
 int
 _gnutls_recv_handshake (gnutls_session_t session, 
                         gnutls_handshake_description_t type,
-                        optional_t optional, gnutls_buffer_st* buf)
+                        unsigned int optional, gnutls_buffer_st* buf)
 {
   int ret;
   handshake_buffer_st hsk;
 
   ret =
-    _gnutls_handshake_io_recv_int (session, type, &hsk);
+    _gnutls_handshake_io_recv_int (session, type, &hsk, optional);
   if (ret < 0)
     {
-      if (optional == OPTIONAL_PACKET && ret == GNUTLS_E_UNEXPECTED_HANDSHAKE_PACKET)
+      if (optional != 0 && ret == GNUTLS_E_UNEXPECTED_HANDSHAKE_PACKET)
         {
           if (buf) _gnutls_buffer_init(buf);
           return 0;
         }
 
-       if (ret == GNUTLS_E_UNEXPECTED_HANDSHAKE_PACKET)
-        _gnutls_audit_log(session, "Received unexpected handshake message '%s' (%d). Expected '%s' (%d)\n",
-         _gnutls_handshake2str(hsk.htype), (int)hsk.htype, _gnutls_handshake2str(type), (int)type);
-
       return gnutls_assert_val_fatal(ret);
     }
 
@@ -1359,7 +1356,7 @@ cleanup:
  * to the session;
  */
 static int
-_gnutls_client_set_ciphersuite (gnutls_session_t session, opaque suite[2])
+_gnutls_client_set_ciphersuite (gnutls_session_t session, uint8_t suite[2])
 {
   uint8_t z;
   uint8_t cipher_suites[MAX_CIPHERSUITE_SIZE];
@@ -1440,7 +1437,7 @@ _gnutls_client_set_ciphersuite (gnutls_session_t session, opaque suite[2])
 /* This function sets the given comp method to the session.
  */
 static int
-_gnutls_client_set_comp_method (gnutls_session_t session, opaque comp_method)
+_gnutls_client_set_comp_method (gnutls_session_t session, uint8_t comp_method)
 {
   int comp_methods_num;
   uint8_t compression_methods[MAX_ALGOS];
@@ -1485,9 +1482,9 @@ _gnutls_client_set_comp_method (gnutls_session_t session, opaque comp_method)
  */
 static int
 _gnutls_client_check_if_resuming (gnutls_session_t session,
-                                  opaque * session_id, int session_id_len)
+                                  uint8_t * session_id, int session_id_len)
 {
-  opaque buf[2 * TLS_MAX_SESSION_ID_SIZE + 1];
+  char buf[2 * TLS_MAX_SESSION_ID_SIZE + 1];
 
   _gnutls_handshake_log ("HSK[%p]: SessionID length: %d\n", session,
                          session_id_len);
@@ -1539,7 +1536,7 @@ _gnutls_client_check_if_resuming (gnutls_session_t session,
  */
 static int
 _gnutls_read_server_hello (gnutls_session_t session,
-                           opaque * data, int datalen)
+                           uint8_t * data, int datalen)
 {
   uint8_t session_id_len = 0;
   int pos = 0;
@@ -1741,10 +1738,10 @@ static int
 _gnutls_send_client_hello (gnutls_session_t session, int again)
 {
   mbuffer_st *bufel = NULL;
-  opaque *data = NULL;
+  uint8_t *data = NULL;
   int pos = 0, type;
   int datalen = 0, ret = 0;
-  opaque rnd[GNUTLS_RANDOM_SIZE];
+  uint8_t rnd[GNUTLS_RANDOM_SIZE];
   gnutls_protocol_t hver;
   gnutls_buffer_st extdata;
   int rehandshake = 0;
@@ -1948,13 +1945,13 @@ static int
 _gnutls_send_server_hello (gnutls_session_t session, int again)
 {
   mbuffer_st *bufel = NULL;
-  opaque *data = NULL;
+  uint8_t *data = NULL;
   gnutls_buffer_st extdata;
   int pos = 0;
   int datalen, ret = 0;
   uint8_t comp;
   uint8_t session_id_len = session->security_parameters.session_id_size;
-  opaque buf[2 * TLS_MAX_SESSION_ID_SIZE + 1];
+  char buf[2 * TLS_MAX_SESSION_ID_SIZE + 1];
 
   datalen = 0;
 
@@ -2047,7 +2044,7 @@ _gnutls_send_hello (gnutls_session_t session, int again)
  * and internals.compression_method.
  */
 int
-_gnutls_recv_hello (gnutls_session_t session, opaque * data, int datalen)
+_gnutls_recv_hello (gnutls_session_t session, uint8_t * data, int datalen)
 {
   int ret;
 
@@ -2083,7 +2080,7 @@ _gnutls_recv_hello (gnutls_session_t session, opaque * data, int datalen)
 
 static int
 _gnutls_recv_hello_verify_request (gnutls_session_t session,
-				   opaque * data, int datalen)
+				   uint8_t * data, int datalen)
 {
   ssize_t len = datalen;
   size_t pos = 0;
@@ -2291,7 +2288,7 @@ _gnutls_recv_supplemental (gnutls_session_t session)
   _gnutls_debug_log ("EXT[%p]: Expecting supplemental data\n", session);
 
   ret = _gnutls_recv_handshake (session, GNUTLS_HANDSHAKE_SUPPLEMENTAL,
-                                OPTIONAL_PACKET, &buf);
+                                1, &buf);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -2463,7 +2460,7 @@ _gnutls_handshake_client (gnutls_session_t session)
           ret =
             _gnutls_recv_handshake (session, 
                   GNUTLS_HANDSHAKE_HELLO_VERIFY_REQUEST,
-                  OPTIONAL_PACKET, NULL);
+                  1, NULL);
           STATE = STATE11;
           IMED_RET ("recv hello verify", ret, 1);
 
@@ -2478,7 +2475,7 @@ _gnutls_handshake_client (gnutls_session_t session)
       ret =
         _gnutls_recv_handshake (session,
                                 GNUTLS_HANDSHAKE_SERVER_HELLO,
-                                MANDATORY_PACKET, NULL);
+                                0, NULL);
       STATE = STATE2;
       IMED_RET ("recv hello", ret, 1);
 
@@ -2519,7 +2516,7 @@ _gnutls_handshake_client (gnutls_session_t session)
         ret =
           _gnutls_recv_handshake (session,
                                   GNUTLS_HANDSHAKE_SERVER_HELLO_DONE,
-                                  MANDATORY_PACKET, NULL);
+                                  0, NULL);
       STATE = STATE6;
       IMED_RET ("recv server hello done", ret, 1);
     case STATE71:
@@ -2569,7 +2566,7 @@ _gnutls_handshake_client (gnutls_session_t session)
 static ssize_t
 send_change_cipher_spec (gnutls_session_t session, int again)
 {
-  opaque* data;
+  uint8_t* data;
   mbuffer_st * bufel;
   int ret;
   
@@ -2669,12 +2666,25 @@ _gnutls_recv_handshake_final (gnutls_session_t session, int init)
   int ret = 0;
   uint8_t ch;
 
+
   switch (STATE)
     {
     case STATE0:
     case STATE30:
-      ret = _gnutls_recv_int (session, GNUTLS_CHANGE_CIPHER_SPEC, -1, &ch, 1, NULL);
       STATE = STATE30;
+
+      /* This is the last flight and peer cannot be sure
+       * we have received it unless we notify him. So we
+       * wait for a message and retransmit if needed. */
+      if (IS_DTLS(session) && !_dtls_is_async(session) && 
+          gnutls_record_check_pending (session) == 0)
+        {
+          ret = _dtls_wait_and_retransmit(session);
+          if (ret < 0)
+            return gnutls_assert_val(ret);
+        }
+
+      ret = _gnutls_recv_int (session, GNUTLS_CHANGE_CIPHER_SPEC, -1, &ch, 1, NULL);
       if (ret <= 0)
         {
           ERR ("recv ChangeCipherSpec", ret);
@@ -2699,10 +2709,19 @@ _gnutls_recv_handshake_final (gnutls_session_t session, int init)
           gnutls_assert ();
           return ret;
         }
-
+        
     case STATE31:
-      ret = _gnutls_recv_finished (session);
       STATE = STATE31;
+
+       if (IS_DTLS(session) && !_dtls_is_async(session) && 
+           gnutls_record_check_pending( session) == 0)
+         {
+           ret = _dtls_wait_and_retransmit(session);
+           if (ret < 0)
+             return gnutls_assert_val(ret);
+         }
+
+      ret = _gnutls_recv_finished (session);
       if (ret < 0)
         {
           ERR ("recv finished", ret);
@@ -2734,7 +2753,7 @@ _gnutls_handshake_server (gnutls_session_t session)
       ret =
         _gnutls_recv_handshake (session,
                                 GNUTLS_HANDSHAKE_CLIENT_HELLO,
-                                MANDATORY_PACKET, NULL);
+                                0, NULL);
       STATE = STATE1;
       IMED_RET ("recv hello", ret, 1);
 
@@ -2892,7 +2911,7 @@ _gnutls_handshake_common (gnutls_session_t session)
 }
 
 int
-_gnutls_generate_session_id (opaque * session_id, uint8_t * len)
+_gnutls_generate_session_id (uint8_t * session_id, uint8_t * len)
 {
   int ret;
 
