@@ -1,6 +1,5 @@
 /*
- * Copyright (C) 2003-2005, 2008, 2010-2012 Free Software Foundation,
- * Inc.
+ * Copyright (C) 2003-2012 Free Software Foundation, Inc.
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -26,8 +25,6 @@
 
 #include <gnutls_int.h>
 
-#ifdef ENABLE_PKI
-
 #include <gnutls_datum.h>
 #include <gnutls_global.h>
 #include <gnutls_errors.h>
@@ -283,7 +280,7 @@ gnutls_x509_crl_set_crt (gnutls_x509_crl_t crl, gnutls_x509_crt_t crt,
                          time_t revocation_time)
 {
   int ret;
-  opaque serial[128];
+  uint8_t serial[128];
   size_t serial_size;
 
   if (crl == NULL || crt == NULL)
@@ -509,4 +506,3 @@ gnutls_x509_crl_privkey_sign (gnutls_x509_crl_t crl, gnutls_x509_crt_t issuer,
   return 0;
 }
 
-#endif /* ENABLE_PKI */
