@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2005, 2008-2012 Free Software Foundation, Inc.
+ * Copyright (C) 2003-2012 Free Software Foundation, Inc.
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -26,8 +26,6 @@
 
 #include <gnutls_int.h>
 
-#ifdef ENABLE_PKI
-
 #include <gnutls_datum.h>
 #include <gnutls_global.h>
 #include <gnutls_errors.h>
@@ -128,7 +126,7 @@ gnutls_x509_crq_import (gnutls_x509_crq_t crq,
    */
   if (format == GNUTLS_X509_FMT_PEM)
     {
-      opaque *out;
+      uint8_t *out;
 
       /* Try the first header */
       result = _gnutls_fbase64_decode (PEM_CRQ, data->data, data->size, &out);
@@ -740,7 +738,7 @@ gnutls_x509_crq_set_version (gnutls_x509_crq_t crq, unsigned int version)
 int
 gnutls_x509_crq_get_version (gnutls_x509_crq_t crq)
 {
-  opaque version[8];
+  uint8_t version[8];
   int len, result;
 
   if (crq == NULL)
@@ -1508,7 +1506,7 @@ gnutls_x509_crq_get_key_usage (gnutls_x509_crq_t crq,
 {
   int result;
   uint16_t _usage;
-  opaque buf[128];
+  uint8_t buf[128];
   size_t buf_size = sizeof (buf);
 
   if (crq == NULL)
@@ -1567,7 +1565,7 @@ gnutls_x509_crq_get_basic_constraints (gnutls_x509_crq_t crq,
 {
   int result;
   unsigned int tmp_ca;
-  opaque buf[256];
+  uint8_t buf[256];
   size_t buf_size = sizeof (buf);
 
   if (crq == NULL)
@@ -2492,5 +2490,3 @@ cleanup:
   return ret;
 }
 
-#endif /* ENABLE_PKI */
-
