@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004, 2006-2011 Free Software Foundation, Inc.
+ * Copyright (C) 2004-2012 Free Software Foundation, Inc.
  * Copyright (C) 2001,2002 Paul Sheer
  * Portions Copyright (C) 2002,2003 Nikos Mavrogiannopoulos
  *
@@ -26,7 +26,7 @@
 #include <config.h>
 
 #include "common.h"
-#include "serv-gaa.h"
+#include "serv-args.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -40,6 +40,7 @@
 #include <list.h>
 #include <netdb.h>
 #include <unistd.h>
+#include <socket.h>
 
 /* Gnulib portability files. */
 #include "progname.h"
@@ -56,32 +57,33 @@
 static int generate = 0;
 static int http = 0;
 static int x509ctype;
-static int debug;
+static int debug = 0;
 
-int verbose;
+unsigned int verbose = 1;
 static int nodb;
 static int noticket;
 int require_cert;
 int disable_client_cert;
 
-char *psk_passwd;
-char *srp_passwd;
-char *srp_passwd_conf;
-char *pgp_keyring;
-char *pgp_keyfile;
-char *pgp_certfile;
-char *x509_keyfile;
-char *x509_certfile;
-char *x509_dsakeyfile;
-char *x509_dsacertfile;
-char *x509_ecckeyfile;
-char *x509_ecccertfile;
-char *x509_cafile;
-char *dh_params_file;
-char *x509_crlfile = NULL;
+const char *psk_passwd = NULL;
+const char *srp_passwd = NULL;
+const char *srp_passwd_conf = NULL;
+const char *pgp_keyring = NULL;
+const char *pgp_keyfile = NULL;
+const char *pgp_certfile = NULL;
+const char *x509_keyfile = NULL;
+const char *x509_certfile = NULL;
+const char *x509_dsakeyfile = NULL;
+const char *x509_dsacertfile = NULL;
+const char *x509_ecckeyfile = NULL;
+const char *x509_ecccertfile = NULL;
+const char *x509_cafile = NULL;
+const char *dh_params_file = NULL;
+const char *x509_crlfile = NULL;
+const char * priorities = NULL;
 
 gnutls_datum_t session_ticket_key;
-static int tcp_server(const char* name, int port);
+static void tcp_server(const char* name, int port);
 
 /* end of globals */
 
@@ -111,8 +113,6 @@ gnutls_psk_server_credentials_t psk_cred = NULL;
 gnutls_anon_server_credentials_t dh_cred = NULL;
 gnutls_certificate_credentials_t cert_cred = NULL;
 
-static gaainfo info;
-
 const int ssl_session_cache = 128;
 
 static void wrap_db_init (void);
@@ -121,6 +121,8 @@ static int wrap_db_store (void *dbf, gnutls_datum_t key, gnutls_datum_t data);
 static gnutls_datum_t wrap_db_fetch (void *dbf, gnutls_datum_t key);
 static int wrap_db_delete (void *dbf, gnutls_datum_t key);
 
+static void cmd_parser (int argc, char **argv);
+
 
 #define HTTP_STATE_REQUEST	1
 #define HTTP_STATE_RESPONSE	2
@@ -250,7 +252,7 @@ static char pkcs3[] =
 static int
 static_dh_params (void)
 {
-  gnutls_datum_t params = { pkcs3, sizeof (pkcs3) };
+  gnutls_datum_t params = { (void*)pkcs3, sizeof (pkcs3) };
   int ret;
 
   if (gnutls_dh_params_init (&dh_params) < 0)
@@ -332,6 +334,9 @@ gnutls_session_t initialize_session (int dtls)
   gnutls_session_t session;
   const char *err;
 
+  if (priorities == NULL)
+    priorities = "NORMAL";
+
   if (dtls)
     gnutls_init (&session, GNUTLS_SERVER|GNUTLS_DATAGRAM);
   else
@@ -353,7 +358,7 @@ gnutls_session_t initialize_session (int dtls)
     gnutls_session_ticket_enable_server (session, &session_ticket_key);
 #endif
 
-  if (gnutls_priority_set_direct (session, info.priorities, &err) < 0)
+  if (gnutls_priority_set_direct (session, priorities, &err) < 0)
     {
       fprintf (stderr, "Syntax error at: %s\n", err);
       exit (1);
@@ -407,7 +412,7 @@ peer_print_info (gnutls_session_t session, int *ret_length,
   char *crtinfo = NULL;
   size_t ncrtinfo = 0;
 
-  if (verbose != 0)
+  if (verbose == 0)
     {
       http_buffer = malloc (len);
       if (http_buffer == NULL)
@@ -420,7 +425,6 @@ peer_print_info (gnutls_session_t session, int *ret_length,
       *ret_length =
         sizeof (DEFAULT_DATA) + sizeof (HTTP_BEGIN) + sizeof (HTTP_END) - 3;
       return http_buffer;
-
     }
 
   if (gnutls_certificate_type_get (session) == GNUTLS_CRT_X509)
@@ -898,18 +902,16 @@ tls_audit_log_func (gnutls_session_t session, const char *str)
   fprintf (stderr, "|<%p>| %s", session, str);
 }
 
-static void gaa_parser (int argc, char **argv);
-
 int
 main (int argc, char **argv)
 {
-  int ret;
+  int ret, mtu, port;
   char name[256];
 
   set_program_name (argv[0]);
+  cmd_parser(argc, argv);
 
 #ifndef _WIN32
-  signal (SIGPIPE, SIG_IGN);
   signal (SIGHUP, SIG_IGN);
   signal (SIGTERM, terminate);
   if (signal (SIGINT, terminate) == SIG_IGN)
@@ -918,12 +920,10 @@ main (int argc, char **argv)
 
   sockets_init ();
 
-  gaa_parser (argc, argv);
-
   if (nodb == 0)
     wrap_db_init ();
 
-  if (info.udp != 0)
+  if (HAVE_OPT(UDP))
     strcpy(name, "UDP ");
   else name[0] = 0;
 
@@ -988,7 +988,6 @@ main (int argc, char **argv)
           printf ("Processed %d CA certificate(s).\n", ret);
         }
     }
-#ifdef ENABLE_PKI
   if (x509_crlfile != NULL)
     {
       if ((ret = gnutls_certificate_set_x509_crl_file
@@ -1003,7 +1002,6 @@ main (int argc, char **argv)
           printf ("Processed %d CRL(s).\n", ret);
         }
     }
-#endif
 
 #ifdef ENABLE_OPENPGP
   if (pgp_keyring != NULL)
@@ -1018,11 +1016,11 @@ main (int argc, char **argv)
         }
     }
 
-  if (pgp_certfile != NULL)
+  if (HAVE_OPT(PGPCERTFILE))
     {
-      if (info.pgp_subkey != NULL)
+      if (HAVE_OPT(PGPSUBKEY))
         ret = gnutls_certificate_set_openpgp_key_file2
-          (cert_cred, pgp_certfile, pgp_keyfile, info.pgp_subkey,
+          (cert_cred, pgp_certfile, pgp_keyfile, OPT_ARG(PGPSUBKEY),
            GNUTLS_OPENPGP_FMT_BASE64);
       else
         ret = gnutls_certificate_set_openpgp_key_file
@@ -1109,10 +1107,10 @@ main (int argc, char **argv)
           GERR (ret);
         }
 
-      if (info.psk_hint)
+      if (HAVE_OPT(PSKHINT))
         {
           ret = gnutls_psk_set_server_credentials_hint (psk_cred,
-                                                        info.psk_hint);
+                                                        OPT_ARG(PSKHINT));
           if (ret)
             {
               fprintf (stderr, "Error setting PSK identity hint.\n");
@@ -1136,13 +1134,22 @@ main (int argc, char **argv)
     gnutls_session_ticket_key_generate (&session_ticket_key);
 #endif
 
-  if (info.udp)
-    return udp_server(name, info.port, info.mtu);
+  if (HAVE_OPT(MTU))
+    mtu = OPT_VALUE_MTU;
+  else mtu = 1300;
+
+  if (HAVE_OPT(PORT))
+    port = OPT_VALUE_PORT;
   else
-    return tcp_server(name, info.port);
+    port = 5556;
+
+  if (HAVE_OPT(UDP))
+    udp_server(name, port, mtu);
+  else
+    tcp_server(name, port);
 }
 
-static int tcp_server(const char* name, int port)
+static void tcp_server(const char* name, int port)
 {
   int n, s;
   char topbuf[512];
@@ -1245,7 +1252,7 @@ static int tcp_server(const char* name, int port)
                                           gl_fd_to_handle (accept_fd));
                 j->handshake_ok = 0;
 
-                if (verbose == 0)
+                if (verbose != 0)
                   {
                     tt = time (0);
                     ctt = ctime (&tt);
@@ -1291,16 +1298,18 @@ static int tcp_server(const char* name, int port)
                 else if (r == 0)
                   {
                     if (gnutls_session_is_resumed (j->tls_session) != 0
-                        && verbose == 0)
+                        && verbose != 0)
                       printf ("*** This is a resumed session\n");
 
-                    if (verbose == 0)
+                    if (verbose != 0)
                       {
                         printf ("\n* Successful handshake from %s\n",
                                 human_addr ((struct sockaddr *)
                                             &client_address, calen, topbuf,
                                             sizeof (topbuf)));
-                        print_info (j->tls_session, NULL, 1);
+                        print_info (j->tls_session, verbose);
+                        if (gnutls_auth_get_type (j->tls_session) == GNUTLS_CRD_CERTIFICATE)
+                          cert_verify(j->tls_session, NULL);
                       }
                     j->handshake_ok = 1;
                   }
@@ -1414,16 +1423,18 @@ static int tcp_server(const char* name, int port)
                 else if (r == 0)
                   {
                     if (gnutls_session_is_resumed (j->tls_session) != 0
-                        && verbose == 0)
+                        && verbose != 0)
                       printf ("*** This is a resumed session\n");
-                    if (verbose == 0)
+                    if (verbose != 0)
                       {
                         printf ("- connection from %s\n",
                                 human_addr ((struct sockaddr *)
                                             &client_address, calen, topbuf,
                                             sizeof (topbuf)));
 
-                        print_info (j->tls_session, NULL, 1);
+                        print_info (j->tls_session, verbose);
+                        if (gnutls_auth_get_type (j->tls_session) == GNUTLS_CRD_CERTIFICATE)
+                          cert_verify(j->tls_session, NULL);
                       }
                     j->handshake_ok = 1;
                   }
@@ -1528,60 +1539,83 @@ static int tcp_server(const char* name, int port)
     wrap_db_deinit ();
   gnutls_global_deinit ();
 
-  return 0;
-
 }
 
-void
-gaa_parser (int argc, char **argv)
+static void cmd_parser (int argc, char **argv)
 {
-  if (gaa (argc, argv, &info) != -1)
+  optionProcess( &gnutls_servOptions, argc, argv);
+
+  disable_client_cert = HAVE_OPT(DISABLE_CLIENT_CERT);
+  require_cert = HAVE_OPT(REQUIRE_CLIENT_CERT);
+  if (HAVE_OPT(DEBUG))
+    debug = OPT_VALUE_DEBUG;
+
+  if (HAVE_OPT(QUIET))
+    verbose = 0;
+
+  if (HAVE_OPT(PRIORITY)) 
+    priorities = OPT_ARG(PRIORITY);
+  
+  if (HAVE_OPT(LIST))
     {
-      fprintf (stderr,
-               "Error in the arguments. Use the --help or -h parameters to get more information.\n");
-      exit (1);
+      print_list(priorities, verbose);
+      exit(0);
     }
 
-  disable_client_cert = info.disable_client_cert;
-  require_cert = info.require_cert;
-  debug = info.debug;
-  verbose = info.quiet;
-  nodb = info.nodb;
-  noticket = info.noticket;
+  nodb = HAVE_OPT(NODB);
+  noticket = HAVE_OPT(NOTICKET);
 
-  if (info.http == 0)
+  if (HAVE_OPT(ECHO))
     http = 0;
-  else
-    http = 1;
+  else http = 1;
 
-  if (info.fmtder == 0)
-    x509ctype = GNUTLS_X509_FMT_PEM;
-  else
+  if (HAVE_OPT(X509FMTDER))
     x509ctype = GNUTLS_X509_FMT_DER;
-
-  if (info.generate == 0)
-    generate = 0;
   else
-    generate = 1;
+    x509ctype = GNUTLS_X509_FMT_PEM;
 
-  dh_params_file = info.dh_params_file;
+  generate = HAVE_OPT(GENERATE);
 
-  x509_certfile = info.x509_certfile;
-  x509_keyfile = info.x509_keyfile;
-  x509_dsacertfile = info.x509_dsacertfile;
-  x509_dsakeyfile = info.x509_dsakeyfile;
-  x509_ecccertfile = info.x509_ecccertfile;
-  x509_ecckeyfile = info.x509_ecckeyfile;
-  x509_cafile = info.x509_cafile;
-  x509_crlfile = info.x509_crlfile;
-  pgp_certfile = info.pgp_certfile;
-  pgp_keyfile = info.pgp_keyfile;
-  srp_passwd = info.srp_passwd;
-  srp_passwd_conf = info.srp_passwd_conf;
+  if (HAVE_OPT(DHPARAMS))
+    dh_params_file = OPT_ARG(DHPARAMS);
 
-  psk_passwd = info.psk_passwd;
+  if (HAVE_OPT(X509KEYFILE))
+    x509_keyfile = OPT_ARG(X509KEYFILE);
+  if (HAVE_OPT(X509CERTFILE))
+    x509_certfile = OPT_ARG(X509CERTFILE);
+
+  if (HAVE_OPT(X509DSAKEYFILE))
+    x509_dsakeyfile = OPT_ARG(X509DSAKEYFILE);
+  if (HAVE_OPT(X509DSACERTFILE))
+    x509_dsacertfile = OPT_ARG(X509DSACERTFILE);
+
+
+  if (HAVE_OPT(X509ECCKEYFILE))
+    x509_ecckeyfile = OPT_ARG(X509ECCKEYFILE);
+  if (HAVE_OPT(X509CERTFILE))
+    x509_ecccertfile = OPT_ARG(X509ECCCERTFILE);
+  
+  if (HAVE_OPT(X509CAFILE))
+    x509_cafile = OPT_ARG(X509CAFILE);
+  if (HAVE_OPT(X509CRLFILE))
+    x509_crlfile = OPT_ARG(X509CRLFILE);
+
+  if (HAVE_OPT(PGPKEYFILE))
+    pgp_keyfile = OPT_ARG(PGPKEYFILE);
+  if (HAVE_OPT(PGPCERTFILE))
+    pgp_certfile = OPT_ARG(PGPCERTFILE);
+
+  if (HAVE_OPT(PGPKEYRING))
+    pgp_keyring = OPT_ARG(PGPKEYRING);
+
+  if (HAVE_OPT(SRPPASSWD))
+    srp_passwd = OPT_ARG(SRPPASSWD);
+  if (HAVE_OPT(SRPPASSWDCONF))
+    srp_passwd_conf = OPT_ARG(SRPPASSWDCONF);
+
+  if (HAVE_OPT(PSKPASSWD))
+    psk_passwd = OPT_ARG(PSKPASSWD);
 
-  pgp_keyring = info.pgp_keyring;
 }
 
 extern void serv_version (void);
